// Entity information
ENT.Type = "brush"
ENT.Base = "base_brush"
	
// Called when the entity first spawns
function ENT:Initialize()
	players_in_box = 0
	switching_maps = 0
	next_check_time = CurTime()
	self_ent = self.Entity
	
	// Get networked variables
	autoswitch_percent = self_ent:GetNetworkedInt("autoswitch_percent") / 100
	autoswitch_time = self_ent:GetNetworkedInt("autoswitch_time")
	next_map = self_ent:GetNetworkedString("next_map")
	local tcl_min = self_ent:GetNetworkedVector("tcl_min")
	local tcl_max = self_ent:GetNetworkedVector("tcl_max")
	
	// Decide the size of the trigger
	local x = tcl_max.x - tcl_min.x
	local y = tcl_max.y - tcl_min.y
	local z = tcl_max.z - tcl_min.z
	
	//Vectors
	local tdml_min = Vector(0-(x/2), 0-(y/2), 0-(z/2))
	local tdml_max = Vector(x/2, y/2, z/2)
	
	//Set  box
	self_ent:PhysicsInitBox(tdml_min, tdml_max)
	self_ent:SetCollisionBounds(tdml_min, tdml_max)
end


// Execute this when a player touches the entity
function ENT:StartTouch(ent)
	if ent and ent:IsValid() and ent:IsPlayer() then
		ent:Lock()
		ent:SetMoveType(0)
		ent:SetTeam(3)
		
		// Freeze vehicle too
		if ent:GetVehicle() and ent:GetVehicle():IsValid() then
			ent:GetVehicle():SetCollisionGroup(11)
			ent:GetVehicle():SetMoveType(0)
		end
		
		// Get players in box/server
		players_in_box = players_in_box + 1
		players_in_server = #player.GetAll()
			
		// Let everyone know that someone entered the loading section
		for _, pl in pairs(player.GetAll()) do
			pl:PrintMessage(HUD_PRINTTALK, ent:Nick().." has entered the loading section ("..string.ToMinutesSeconds(CurTime() - pl.start_time)..") ["..players_in_box.." of "..players_in_server.."].")
		end
	end
end 


// Checks to see if we should go to the next map
function ENT:Think()
	if next_check_time < CurTime() then
		players_in_server = #player.GetAll()
		
		// Are we ready to change the map?
		if switching_maps == 0 and players_in_server > 0 and players_in_box >= (players_in_server * autoswitch_percent) then
			switching_maps = 1
			
			// Let remaining people know
			for _, pl in pairs(player.GetAll()) do
				if pl:Team() != 3 then
					pl:PrintMessage(HUD_PRINTTALK, "You have "..autoswitch_time.." seconds to complete the level before the map changes.")
				else
					pl:PrintMessage(HUD_PRINTTALK, "Autoswitch timer started, you only have to wait "..autoswitch_time.." more seconds.")
				end
			end
				
			// Start the countdown timer
			timer.Simple(autoswitch_time, game.ConsoleCommand, "changegamemode "..next_map.." hl2-campaign\n")
		end
		
		// Next check time
		next_check_time = CurTime() + 1
	end
end


// These are needed to keep console from flipping out
function ENT:Touch()
end


// If the player decides to leave our box
function ENT:EndTouch(ent)
end 