// Send resources to client
resource.AddFile("materials/hl2c_nav_marker.vmt")
resource.AddFile("materials/hl2c_nav_marker.vtf")


// Send required lua files to client
AddCSLuaFile("cl_init.lua")
AddCSLuaFile("config.lua")
AddCSLuaFile("shared.lua")


// Include needed files
include("tables.lua")
include("shared.lua")


// Called when we need to change the gamemode difficulty
function DoConCommand(pl, command, args)
	if command == "hl2c_crosshair" then
		if args[1] == 1 then
			pl:SetNetworkedInt("ch_enabled", 1)
			pl:CrosshairEnable()
		else
			pl:SetNetworkedInt("ch_enabled", 0)
			pl:CrosshairDisable()
		end
	end
end
concommand.Add("hl2c_crosshair", DoConCommand)


// Called when the player attempts to suicide
function GM:CanPlayerSuicide(pl)
	if #player.GetAll() > 1 then
		pl:PrintMessage(HUD_PRINTTALK, "Type !moveto [playername] in chat to be teleported to that player.")
		return false
	else
		return true
	end
end 


// Called when the player dies
function GM:DoPlayerDeath(pl, attacker, dmginfo) 
	pl:CreateRagdoll()
	pl:SetTeam(2)
	pl:AddDeaths(1)
	pl:Freeze(false)
	
	// Remove their vehicle
	if pl.vehicle and pl.vehicle:IsValid() then
		pl.vehicle:Remove()
		pl.vehicle = nil
	end
	 
	// Grab player's weapons
	pl.death_weapons = {}
	for _, death_weapon in pairs(pl:GetWeapons()) do
		table.insert(pl.death_weapons, death_weapon:GetClass())
	end
	
	// Next spawn time
	pl:SetNetworkedInt("next_spawn_time", CurTime() + hl2c_respawn_time)
end


// Called when map entities spawn
function GM:EntityKeyValue(ent, key, value)
	if ent:GetClass() == "trigger_changelevel" and key == "map" then
		table.insert(tcl_maps, value)
	elseif ent:GetClass() == "ambient_generic" and key == "message" then
		table.insert(ag_messages, value)
	end
end


// Called when an entity has received damage	  
function GM:EntityTakeDamage(ent, inflictor, attacker, amount, dmginfo)
	if ent and ent:IsValid() and !ent:IsPlayer() and attacker and attacker:IsValid() and attacker:IsPlayer() and attacker:GetActiveWeapon():GetClass() == "weapon_crowbar" then
		local ent_health = ent:Health()
		local ent_health_new = ent_health * 1.8
		ent:SetHealth(ent_health_new)
	end
end


// Can we punt other player's vehicles?
function GM:GravGunPunt(pl, ent) 
 	if ent and ent:IsVehicle() and ent != pl.vehicle then
		return false
	else
		return true
	end	
end 


// Called immediately after starting the gamemode  
function GM:Initialize()
	current_map = game.GetMap()
	next_map = map_setup[current_map][4]
	tcl_maps = {}
	ag_messages = {}
	first_checkpoint = 1
	next_checkpoint_time = CurTime() + hl2c_checkpoint_time
	difficulty = 1 // Initial value, this will change when first checkpoint is made
	
	// We want regular fall damage and the ai to attack players
	game.ConsoleCommand("mp_falldamage 1\n")
	game.ConsoleCommand("ai_ignoreplayers 0\n")
	
	// Jeep
	local jeep = {
		Name = "Jeep",
		Class = "prop_vehicle_jeep",
		Model = "models/vehicle.mdl",
		KeyValues = {	
			vehiclescript =	"scripts/vehicles/jeep_test.txt",
		}
	}
	list.Set("Vehicles", "Jeep", jeep)
	
	// Airboat
	local airboat = {
		Name = "Airboat Gun",
		Class = "prop_vehicle_airboat",
		Category = Category,
		Model = "models/airboat.mdl",
		KeyValues = {
			vehiclescript = "scripts/vehicles/airboat.txt",
			EnableGun = 0
		}
	}
	list.Set("Vehicles", "Airboat", airboat)
	
	// Airboat w/gun
	local airboat_gun = {
		Name = "Airboat Gun",
		Class = "prop_vehicle_airboat",
		Category = Category,
		Model = "models/airboat.mdl",
		KeyValues = {
			vehiclescript = "scripts/vehicles/airboat.txt",
			EnableGun = 1
		}
	}
	list.Set("Vehicles", "Airboat Gun", airboat_gun)
	
	// Jalopy
	local jalopy = {
		Name = "Jalopy",
		Class = "prop_vehicle_jeep",
		Model = "models/vehicle.mdl",
		KeyValues = {	
			vehiclescript =	"scripts/vehicles/jalopy.txt",
		}
	}
	list.Set("Vehicles", "Jalopy", jalopy)
		
	// Change the next map if we need to
	skip_one = 0
	if file.Exists("hl2-campaign/d1_town_03.txt") then
		if current_map == "d1_town_03" then
			skip_one = 1
		elseif current_map == "d1_town_02" then
			next_map = "d1_town_02a"
		end
	elseif file.Exists("hl2-campaign/d2_coast_08.txt") and current_map == "d2_coast_07" then
		next_map = "d2_coast_09"
	elseif file.Exists("hl2-campaign/ep2_outland_04.txt") and current_map == "ep2_outland_02" then
		next_map = "ep2_outland_05"
	end
	
	// Going to episode 1 or 2?
	if current_map == "d3_breen_01" and hl2c_play_ep1 == 1 then
		next_map = "ep1_citadel_00"
	elseif current_map == "ep1_c17_06" and hl2c_play_ep2 == 1 then
		next_map = "ep2_outland_01"
	end
	
	// Create / delete data files when we have reached or passed certain maps
	if current_map == "d1_town_03" then
		if !file.Exists("hl2-campaign/d1_town_03.txt") then
			file.Write("hl2-campaign/d1_town_03.txt", "We have been to d1_town_03.")
		end
	elseif current_map == "ep2_outland_04" then
		if !file.Exists("hl2-campaign/ep2_outland_04.txt") then
			file.Write("hl2-campaign/ep2_outland_04.txt", "We have been to ep2_outland_04.")
		end
	elseif current_map == "d1_town_02a" then
		if file.Exists("hl2-campaign/d1_town_03.txt") then
			file.Delete("hl2-campaign/d1_town_03.txt")
		end
	elseif current_map == "d2_coast_09" then
		if file.Exists("hl2-campaign/d2_coast_08.txt") then
			file.Delete("hl2-campaign/d2_coast_08.txt")
		end
	elseif current_map == "ep2_outland_05" then
		if file.Exists("hl2-campaign/ep2_outland_04.txt") then
			file.Delete("hl2-campaign/ep2_outland_04.txt")
		end
	end
end


// Called as soon as all map entities have been spawned 
function GM:InitPostEntity()
	if !map_setup[current_map] then
		game.ConsoleCommand("changegamemode d1_trainstation_01 hl2-campaign\n")
		return
	end
	
	// Info player start stuff
	local ips_count = 1
	local ips_number = #ents.FindByClass("info_player_start") + 1
	
	// Find out if we need to create our own info_player_start, if so, then setup the posistion for it, if not get the start_ips_number from our master table
	if current_map == "d1_town_02" and file.Exists("hl2-campaign/d1_town_03.txt") then
		map_setup[current_map][1] = 6
		custom_ips_pos = Vector(-3755, -28, -3366)
	elseif current_map == "d2_coast_07" and file.Exists("hl2-campaign/d2_coast_08.txt") then
		custom_ips_pos = Vector(2999, 3684, 1609)
	elseif current_map == "d2_coast_08" then
		custom_ips_pos = Vector(3328, 668, 1608)	
	elseif current_map == "d3_breen_01" then
		custom_ips_pos = Vector(-2450, -1292, 654)
	elseif current_map == "ep1_citadel_00" then
		custom_ips_pos = Vector(-9007, 5792, -59)
	elseif current_map == "ep2_outland_02" and file.Exists("hl2-campaign/ep2_outland_04.txt") then
		map_setup[current_map][1] = 6
		custom_ips_pos = Vector(-3117, -9453, -3100)
	elseif current_map == "ep2_outland_10" then
		custom_ips_pos = Vector(4689, -11100, -943)
	else
		ips_number = map_setup[current_map][2]
	end
	
	// Spawn the custom info_player_start entity if we need to
	if custom_ips_pos then
		local custom_ips = ents.Create("info_player_start")
		custom_ips:SetPos(custom_ips_pos)
		custom_ips:Spawn()
	end
	
	// Remove all the spawn points except for the one we want to use
	for _, ips in pairs(ents.FindByClass("info_player_start")) do
		if ips_count != ips_number then
			ips:Remove()
		end
		ips_count = ips_count + 1
	end
	
	// Create my custom trigger_delayload entities
	for tcl_count, tcl in pairs(ents.FindByClass("trigger_changelevel")) do
		if tcl_maps[tcl_count] == next_map then
			if skip_one == 1 then
				skip_one = 0
			else
				local tcl_min, tcl_max = tcl:WorldSpaceAABB() 
				
				// Any maps that need custom loading points?
				if current_map == "d1_canals_13" then
					tcl_min = Vector(-1099, -3848, -371)
					tcl_max = Vector(-497, -4200, -133)
				elseif current_map == "d2_prison_08" then
					tcl_min = Vector(-964, -953, 910)
					tcl_max = Vector(-854, -1103, 1081)
				elseif current_map == "ep1_c17_00a" then
					tcl_min = Vector(4520, 3638, 2000)
					tcl_max = Vector(4663, 3516, 2072)
				end
				
				// Create our new tdml
				tdml_pos = tcl_max - ((tcl_max - tcl_min) / 2)
				tdml = ents.Create("trigger_delaymapload")
				tdml:SetPos(tdml_pos)
				tdml:SetNetworkedString("next_map", tcl_maps[tcl_count])  
				tdml:SetNetworkedVector("tcl_min", tcl_min)
				tdml:SetNetworkedVector("tcl_max", tcl_max)
				tdml:SetNetworkedInt("autoswitch_time", hl2c_autoswitch_time)
				tdml:SetNetworkedInt("autoswitch_percent", hl2c_autoswitch_percent)
				tdml:Spawn()
				
				// Set global
				SetGlobalVector("tdml_pos", tdml_pos)
			end
		end
		tcl:Remove()
	end
	
	// Find entity types that we need to delete/fix
	local func_movelinears = ents.FindByClass("func_movelinear")
	local func_brushes = ents.FindByClass("func_brush")
	local trigger_onces = ents.FindByClass("trigger_once")
	local trigger_multiples = ents.FindByClass("trigger_multiple")
	local ambient_generics = ents.FindByClass("ambient_generic")
	
	// Now delete/fix them
	if current_map == "d1_canals_05" then
		func_movelinears[1]:Remove()
	elseif current_map == "d1_town_02a" then
		func_movelinears[1]:Remove()
	elseif current_map == "d2_coast_07" and file.Exists("hl2-campaign/d2_coast_08.txt") then
		func_brushes[1]:Remove()
		func_brushes[2]:Remove()
		func_brushes[3]:Remove()
	elseif current_map == "d2_prison_07" then
		trigger_onces[8]:Remove()
	elseif current_map == "d3_c17_06b" then
		local func_door_rotatings = ents.FindByClass("func_door_rotating")
		for _, fdr in pairs(func_door_rotatings) do
			if fdr:GetName() == "long_plank_1_rotator" then
				fdr:SetMoveType(0)
				fdr:SetAngles(fdr:GetAngles() + Vector(0, -45, 0))
			end
		end
	end
	
	// Remove all triggers that cause the game to "end"
	for _, tm in pairs(trigger_multiples) do
		if tm:GetName() == "fall_trigger" then
			tm:Remove()
		end
	end
	
	// If we are a dedicated server, remove all music because it causes the server to crash
	if isDedicatedServer() then
		for b, am_gen in pairs(ambient_generics) do
			if string.find(ag_messages[b], "song") then
				am_gen:Remove()
			end
		end
	end
end 


// Called when an NPC dies
function GM:OnNPCKilled(npc, killer, weapon)
	if killer and killer:IsValid() and killer:IsVehicle() and killer:GetDriver():IsPlayer() then
		killer = killer:GetDriver()
	end
	
	// If the killer is a player then decide what to do with their points
	if killer and killer:IsValid() and killer:IsPlayer() and npc and npc:IsValid() and (table.HasValue(godlike_npcs, npc:GetClass()) or table.HasValue(friendly_npcs, npc:GetClass())) then
		killer:AddFrags(-500)
	elseif killer and killer:IsValid() and killer:IsPlayer() then
		killer:AddFrags(1)
	end
end


// Called when a player tries to pickup a weapon
function GM:PlayerCanPickupWeapon(pl, weapon) 
	if weapon:GetClass() == "weapon_stunstick" or (weapon:GetClass() == "weapon_physgun" and !pl:IsAdmin() and !pl:IsSuperAdmin()) then
		return false
	else
		return true
	end
end


// Called when the player is waiting to respawn
function GM:PlayerDeathThink(pl)
	if pl:GetNetworkedInt("next_spawn_time") > CurTime() then
		return
	else
		pl:Spawn()
	end
end 


// Called when a player entered a vehicle
function GM:PlayerEnteredVehicle(pl, vehicle, role) 
	if pl.vehicle != vehicle then
		pl.vehicle = vehicle
		
		// Now make sure no one else owns that vehicle, if they do, they don't own it anymore
		for _, pl2 in pairs(player.GetAll()) do
			if pl != pl2 and pl.vehicle == pl2.vehicle then
				pl2.vehicle = nil
			end
		end
	end
end


// Called just before the player's first spawn 
function GM:PlayerInitialSpawn(pl)
	pl.start_time = CurTime()
	pl:SetNetworkedInt("show_nav", 1)
	
	// Decide where to spawn vehicle
	if current_map == "d1_canals_05" or current_map == "d2_coast_01" then
		pl.spawn_vehicle_on_pl = 0
	else
		pl.spawn_vehicle_on_pl = 1
	end
end 


// Give the player the default spawning weapons/ammo
function GM:PlayerLoadout(pl)
	pl:StripWeapons()
	
	// Do they have any death weapons
	if pl.death_weapons then
		for _, death_weapon in pairs(pl.death_weapons) do
			pl:Give(death_weapon)
		end
	end
	
	// Now, get the correct weapons table for ep1/ep2
	if string.find(current_map, "ep1") then
		hl2c_weapons = hl2c_ep1_weapons
	elseif string.find(current_map, "ep2") then
		hl2c_weapons = hl2c_ep2_weapons
	else
		hl2c_weapons = hl2c_hl2_weapons
	end
	
	// If we have no weapons, give them to em
	local wep_count_end = map_setup[current_map][1]
	if wep_count_end == -1 then
		pl:Give("weapon_physcannon")
		pl:Give("weapon_ar2")
	elseif #pl:GetWeapons() < wep_count_end then
		for wep_count = 1, wep_count_end do
			pl:Give(hl2c_weapons[wep_count][1])
		end
	end
	
	// Remove all the ammo the got from the weapons
	pl:RemoveAllAmmo()
		
	// Now give the weapon correct ammo
	for a, wep in pairs(pl:GetWeapons()) do
		for _, wep_info in pairs(hl2c_weapons) do
			if wep_info[1] == wep:GetClass() then
				pl:GiveAmmo(wep_info[2], wep:GetPrimaryAmmoType())
			end
		end
	end
	
	// Gravity gun is needed in certain spots (temp until we get use pickup)
	if current_map == "d1_trainstation_01" or current_map == "d1_trainstation_05" or current_map == "d1_canals_06" then
		pl:Give("weapon_physcannon")
	end
	
	// Lastly give physgun to admins
	if hl2c_physgun == 1 and (pl:IsAdmin() or pl:IsSuperAdmin()) then
		pl:Give("weapon_physgun")
	end
end


// Called when the player attempts to noclip
function GM:PlayerNoClip(pl)
	if pl:IsAdmin() and hl2c_noclip == 1 then
		return true
	else
		return false
	end
end 


// Called when a player says something
function GM:PlayerSay(pl, text, toall) 
	local text_explode = string.Explode(" ", text)
	
	// What do we got
	if text_explode[1] == "!nextmap" and (pl:IsAdmin() or pl:IsSuperAdmin()) then 
		game.ConsoleCommand("changegamemode "..next_map.." hl2-campaign\n")
		return ""
	elseif text_explode[1] == "!restartmap" and (pl:IsAdmin() or pl:IsSuperAdmin()) then 
		game.ConsoleCommand("changegamemode "..current_map.." hl2-campaign\n")
		return ""
	elseif text_explode[1] == "!moveto" and text_explode[2] then
		if pl:InVehicle() then 
			pl:PrintMessage(HUD_PRINTTALK, "You must be out of your vehicle in order to use this command!")
			return ""
		end
		
		// Did the player move yet?
		pl.moved = 0
		
		// Find a player?
		for _, pl2 in pairs(player.GetAll()) do
			if pl.moved == 0 and string.find(string.lower(pl2:Name()), string.lower(text_explode[2])) and pl2:Alive() then
				pl.moved = 1
				pl:SetPos(pl2:GetPos())
			end
		end
		
		// Don't display this text
		return ""
	else
		return text
	end
end


// Select the player spawn
function GM:PlayerSelectSpawn(pl)
	local spawn_points = ents.FindByClass("info_player_start")
	selected_spawn = spawn_points[#spawn_points]
	return selected_spawn
end 


// Set the player model
function GM:PlayerSetModel(pl) 
	if !pl.model_name then
		local rand_model_num = math.random(1, #hl2c_player_models)
		pl.model_name = hl2c_player_models[rand_model_num]
	end
	
	// Precache and set the model
	util.PrecacheModel(pl.model_name)
	pl:SetModel(pl.model_name) 
end


// Called when a player spawns 
function GM:PlayerSpawn(pl)
	pl:SetTeam(1)
	pl:SetCollisionGroup(11)
	
	// Networked integers
	pl:SetNetworkedInt("ch_enabled", 1)
	pl:SetNetworkedInt("next_spawn_time", 0)
	pl:SetNetworkedInt("energy", 100)
	
	// Player aux bar variables
	pl.next_er_time = 0
	pl.next_sh_time = 0
	pl.health_removed = 0
	
	// Need to make sure the player doesn't get spammed with weapons
	pl.given_weapons = {}
	
	// Speed, loadout, and model
	GAMEMODE:SetPlayerSpeed(pl, 190, 320)
	GAMEMODE:PlayerLoadout(pl)
 	GAMEMODE:PlayerSetModel(pl)
end


// Called when a player uses something
function GM:PlayerUse(pl, ent)
	if current_map == "d2_coast_08" and !file.Exists("hl2-campaign/d2_coast_08.txt") and ent:GetClass() == "func_door" then
		file.Write("hl2-campaign/d2_coast_08.txt", "We have been to d2_coast_08 and pressed the button.")
	end
	
	// Keep players from crashing the server.
	if ent:GetName() == "telescope_button" then
		return false
	else
		return true
	end
end


// Called every time a player does damage to an npc
function GM:ScaleNPCDamage(npc, hitgroup, dmginfo)
	if table.HasValue(godlike_npcs, npc:GetClass()) or (dmginfo:GetAttacker():IsPlayer() and table.HasValue(friendly_npcs, npc:GetClass())) then
		dmginfo:ScaleDamage(0)
	else
		if hitgroup == HITGROUP_HEAD then
			hitgroup_scale = 3
		else
			hitgroup_scale = 1
		end
		
		// Calculate the damage
		local dmg_scale = hitgroup_scale / difficulty
		dmginfo:ScaleDamage(dmg_scale)
	end
end


// Scale the damage based on being shot in a hitbox 
function GM:ScalePlayerDamage(pl, hitgroup, dmginfo)
	if hitgroup == HITGROUP_HEAD then
		hitgroup_scale = 3
	else
		hitgroup_scale = 1
	end
	
	// Calculate the damage
	local dmg_scale = hitgroup_scale * difficulty
	dmginfo:ScaleDamage(dmg_scale)
end 


// Called when player presses help key
function GM:ShowHelp(pl)
	if pl:GetNetworkedInt("show_help") == 1 then
		pl:SetNetworkedInt("show_help", 0)
	else
		pl:SetNetworkedInt("show_help", 1)
	end
end


// Called when a player presses show team key
function GM:ShowTeam(pl)
	if pl:GetNetworkedInt("show_nav") == 1 then
		pl:SetNetworkedInt("show_nav", 0)
	else
		pl:SetNetworkedInt("show_nav", 1)
	end
end


// Called when player wants a vehicle
function GM:ShowSpare1(pl)
	if pl:Team() != 1 then return end
	
	// Remove current vehicle
	if pl.vehicle and pl.vehicle:IsValid() then
		pl.vehicle:Remove()
		pl.vehicle = nil
	end
	
	// Decide if we should give the player a vehicle
	if map_setup[current_map][3] == 1 then
		vehicle_name = "Airboat"
	elseif map_setup[current_map][3] == 2 then
		vehicle_name = "Airboat Gun"
	elseif map_setup[current_map][3] == 3 then
		vehicle_name = "Jeep"
	elseif map_setup[current_map][3] == 4 then
		vehicle_name = "Jalopy"
	end
	
	// Spawn the vehicle
	if vehicle_name then
		local vehicles = list.Get("Vehicles")
		local vehicle = vehicles[vehicle_name]
		
		// Create the new entity
		new_vehicle = ents.Create(vehicle.Class)
		new_vehicle:SetModel(vehicle.Model)
		
		// Set keyvalues
		for a, b in pairs(vehicle.KeyValues) do
			new_vehicle:SetKeyValue(a, b)
		end		
		
		// Set pos/angle and spawn
		new_vehicle:SetPos(pl:GetPos() + pl:GetAngles():Forward() * 180)
		new_vehicle:SetAngles(pl:GetAngles():Up())
		new_vehicle:Spawn()
		new_vehicle:Activate()
		
		// This is their new vehicle
		pl.vehicle = new_vehicle
	else
		pl:PrintMessage(HUD_PRINTTALK, "You can't spawn a vehicle on this map.")
		pl.vehicle = nil
	end
end


// Called when player wants to removed their vehicle
function GM:ShowSpare2(pl)
	if pl.vehicle and pl.vehicle:IsValid() then
		pl.vehicle:Remove()
	else
		pl:PrintMessage(HUD_PRINTTALK, "You don't have a vehicle to remove.")
	end
	pl.vehicle = nil
end


// Called every frame 
function GM:Think() 
	for _, pl in pairs(player.GetAll()) do
		if pl:Team() == 1 and !pl:InVehicle() then
			for _, pl3 in pairs(player.GetAll()) do
				if pl != pl3 and pl3:Team() == 1 and !pl3:InVehicle() and pl3:GetActiveWeapon():IsValid() and !pl:HasWeapon(pl3:GetActiveWeapon():GetClass()) and pl3:GetActiveWeapon():GetClass() != "weapon_physgun" and !table.HasValue(pl.given_weapons, pl3:GetActiveWeapon():GetClass()) then
					pl:Give(pl3:GetActiveWeapon():GetClass())
					table.insert(pl.given_weapons, pl3:GetActiveWeapon():GetClass())
				end
			end
		end
		
		// Sprinting and water level
		if pl:Alive() and pl.next_er_time < CurTime() then
			if pl:Team() != 3 and pl:GetVelocity():Length() > 315 or pl:WaterLevel() == 3 then
				pl.next_er_time = CurTime() + 0.1
				local cur_energy = pl:GetNetworkedInt("energy")
				if cur_energy > 0 then
					local new_energy = cur_energy - 1
					pl:SetNetworkedInt("energy", new_energy)
				end
			elseif pl:GetVelocity():Length() < 315 then
				pl.next_er_time = CurTime() + 0.1
				local cur_energy = pl:GetNetworkedInt("energy")
				if cur_energy < 100 then
					local new_energy = cur_energy + .5
					pl:SetNetworkedInt("energy", new_energy)
				end
			end
		end
		
		// Now check if they have enough energy 
		if pl:GetNetworkedInt("energy") < 2 then
			pl:SprintDisable()
			
			// Now remove health if underwater
			if pl:WaterLevel() == 3 and pl.next_sh_time < CurTime() then
				pl.next_sh_time = CurTime() + 1
				local cur_health = pl:Health()
				pl:SetHealth(cur_health - 10)
				if pl:Alive() and pl:Health() < 1 then
					pl:Kill()
				end
				pl.health_removed = pl.health_removed + 10
			end				
		elseif pl:GetNetworkedInt("energy") >= 15 then
			pl:SprintEnable()
		end
		
		// Give back health if we can
		if pl:WaterLevel() <= 2 and pl.next_sh_time < CurTime() and pl.health_removed > 0 then
			pl.next_sh_time = CurTime() + 1
			local cur_health = pl:Health()
			pl:SetHealth(cur_health + 10)
			pl.health_removed = pl.health_removed - 10
		end
	end
	
	// Create checkpoint every few seconds
	if next_checkpoint_time < CurTime() and #player.GetAll() > 0 then
		next_checkpoint_time = CurTime() + hl2c_checkpoint_time
		local all_players = player.GetAll()
		local rand_player = all_players[math.random(1, #all_players)]
		
		// Create a new spawn
		if rand_player and rand_player:Alive() and rand_player:IsOnGround() and !rand_player:Crouching() and rand_player:WaterLevel() <= 1 then
			if first_checkpoint == 1 then
				first_checkpoint = 0
			else
				ipss = ents.FindByClass("info_player_start")
				ipss[#ipss]:Remove()
			end
				
			// Create the spawn
			ips = ents.Create("info_player_start")
			ips:SetPos(rand_player:GetPos())
			ips:Spawn()
		end
		
		// Change the difficulty according to number of players
		difficulty = (#player.GetAll() + 1 / 2)
		if difficulty and difficulty > 3 then
			difficulty = 3
		end
	end
end 